@@ -26,9 +26,12 @@
 #include "config.h"
 
 #include <libosso-abook/osso-abook.h>
-#include <qrencode.h>
+#include <hildon/hildon.h>
+#include <hildon/hildon-fm.h>
 #include <cairo.h>
 
+#include "qrcode_export.h"
+
 #define BLOCK_SIZE (12)
 #define PADDING (10)
 #define MIN_SIZE (480)
@@ -53,102 +56,56 @@ qrcode_menu_extension_init(QRcodeMenuExtension *ext)
 {
 }
 
-static cairo_surface_t *
-gqre_qrcode_to_cairo(QRcode *q, gint isize, gdouble padding)
-{
-cairo_surface_t *surface;
-cairo_t *cr;
-gint x,y;
-gdouble rs;
-guchar *qb;
-
-surface=cairo_image_surface_create(CAIRO_FORMAT_RGB24, isize, isize);
-cr=cairo_create(surface);
-
-cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
-
-cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
-cairo_paint(cr);
-
-qb=q->data;
-rs=(((gdouble)isize-2.0*padding)/(gdouble)q->width);
-
-for(y=0; y<q->width; y++) {
-	gdouble rx,ry;
-
-	ry=padding+y*rs;
-	for(x=0; x<q->width; x++) {
-		if (*qb & 1)
-			cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
-		else 
-			cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
-
-		rx=padding+x*rs;
-
-		cairo_rectangle(cr, rx, ry, rs, rs);
-#if 1
-		cairo_fill(cr);
-#else
-		cairo_fill_preserve(cr);
-		cairo_stroke(cr);
-#endif
-		qb++;
-	}
-}
-
-cairo_destroy(cr);
-cairo_surface_flush(surface);
-return surface;
-}
-
 static void
 contact_to_qrcode_cb(GtkWidget *widget, gpointer user_data)
 {
 OssoABookMenuExtension *ext=user_data;
 OssoABookContact *contact;
+GtkWidget *dialog;
 const gchar *contact_name;
-gchar *vcard, *file, *basefile, *qrfile;
+gchar *vcard, *file, *basefile, *dir;
 cairo_surface_t *cs;
-gint isize=MIN_SIZE, ts;
-QRcode *q;
 
 contact=osso_abook_menu_extension_get_contact(ext);
 
-if (!contact)
-	return;
-
-vcard=osso_abook_contact_to_string(contact, EVC_FORMAT_VCARD_30, FALSE);
-if (!vcard)
-	return;
-
-basefile=osso_abook_contact_get_basename(contact, TRUE);
-if (!basefile) {
-	g_free(vcard);
+if (!contact) {
+	hildon_banner_show_information(NULL, NULL, "Failed to get contact.");
 	return;
 }
 
-file=g_strdup_printf("qrcode_%s.png", basefile);
-qrfile=g_build_filename(g_get_home_dir(), "MyDocs", file, NULL);
-g_debug("QREF: %s", qrfile);
+vcard=osso_abook_contact_to_string(contact, EVC_FORMAT_VCARD_30, FALSE);
+if (!vcard) {
+	hildon_banner_show_information(NULL, NULL, "Failed to get contact vCard.");
+	return;
+}
 
-contact_name=osso_abook_contact_get_display_name (contact);
-q=QRcode_encodeString(vcard, 0, QR_ECLEVEL_M, QR_MODE_8, 1);
+basefile=osso_abook_contact_get_basename(contact, TRUE);
+file=g_strdup_printf("qrcode_%s.png", basefile ? basefile : "unknown_contact");
+dir=g_build_filename(g_get_home_dir(), "MyDocs", NULL);
 
-ts=q->width*BLOCK_SIZE;
-isize=CLAMP(ts, MIN_SIZE, MAX_SIZE);
+dialog=hildon_file_chooser_dialog_new(NULL, GTK_FILE_CHOOSER_ACTION_SAVE);
+gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog), file);
+gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), dir);
 
-cs=gqre_qrcode_to_cairo(q, isize, PADDING);
-QRcode_free(q);
-if (cairo_surface_write_to_png(cs, qrfile)==CAIRO_STATUS_SUCCESS)
-	hildon_banner_show_informationf(NULL, NULL, "QR Code exported for contact:\n %s", contact_name);
-else
-	hildon_banner_show_information(NULL, NULL, "QR Code export failed!");
+if (gtk_dialog_run(GTK_DIALOG(dialog))==GTK_RESPONSE_OK) {
+	gchar *fname;
+	
+	fname=gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+	contact_name=osso_abook_contact_get_display_name(contact);
+	cs=gqre_char_to_qrcode_to_cairo(vcard, MIN_SIZE, MAX_SIZE, PADDING);
 
-cairo_surface_destroy(cs);
+	if (cairo_surface_write_to_png(cs, fname)==CAIRO_STATUS_SUCCESS)
+		hildon_banner_show_informationf(NULL, NULL, "QR Code exported for contact:\n %s", contact_name);
+	else
+		hildon_banner_show_information(NULL, NULL, "QR Code export failed!");
 
+	g_free(fname);
+	cairo_surface_destroy(cs);
+}
+gtk_widget_destroy(dialog);
 g_free(vcard);
 g_free(file);
-g_free(qrfile);
+g_free(dir);
 }
 
 /* Entries to show in the menu, the values are:
@@ -181,7 +138,7 @@ OssoABookMenuExtensionClass *menu_extension_class;
 menu_extension_class = OSSO_ABOOK_MENU_EXTENSION_CLASS (cls);
 menu_extension_class->get_n_menu_entries = get_n_menu_entries;
 menu_extension_class->get_menu_entries = get_menu_entries;
-menu_extension_class->name = OSSO_ABOOK_MENU_NAME_CONTACT_VIEW;
+menu_extension_class->name = OSSO_ABOOK_MENU_NAME_CONTACT_VIEW ";" OSSO_ABOOK_MENU_NAME_MECARD_VIEW;
 }
 
 static void
